'''
File consists of several classes for the different elements of a device.
'''
from __future__ import annotations
import uuid
from typing import Any, Union, Literal, TypedDict, TypeVar, Type, List, Optional, Dict


class LegoItem:
    def __init__(self, item_number: int, mass: float, delivery_time: int, **kwargs) -> None:
        # , *args, **kwargs not handling additional/optional specs right now
        self.id: uuid.UUID = uuid.uuid4()
        self.properties: dict = kwargs
        self.item_number: int = item_number
        self.mass: float = mass
        self.delivery_time: int = delivery_time
        # TODO: Set parent directly and not via id? This would allow for easier traversal of the tree
        #   Currently there is no way to search for parts and components in tree by id.
        self.parent_id = None  # This will be set when added to a component

    def __str__(self):
        return (
            f"Item(id={self.id}, item_number={self.item_number}, "
            f"mass={self.mass}, delivery_time={self.delivery_time}, "
            f"parent_id={self.parent_id})"
        )

    def __repr__(self):
        return f"Lego Item [{self.id}]"

class LegoComponent:
    def __init__(self, items=None | LegoItem | list[LegoItem],
                 components=None) -> None:  # TODO: type hinting
        self.id = uuid.uuid4()
        if (isinstance(items, list) and items
           and isinstance(items[0], LegoItem)):
            self.items = items
        elif isinstance(items, LegoItem):
            self.items = [items]
        else:
            self.items = []  # must be a list for now,
        if components is None:
            self.components = []
        elif isinstance(components, list):
            self.components = components
        else:
            self.components = [components]  # Not checking correct type
        self.parent_id = None  # This will be set when added to a component

    def add_item(self, item: LegoItem | List[LegoItem]) -> None:
        if isinstance(item, list):
            self.items.extend(item)
        else:
            self.items.append(item)

    def add_component(self, component: LegoComponent | List[LegoComponent]
                      ) -> None:
        if isinstance(component, list):
            self.components.extend(component)
        else:
            self.components.append(component)

    def add(self, element) -> None:
        if isinstance(element, LegoItem):
            self.items.append(element)
        elif isinstance(element, LegoComponent):
            self.components.append(element)
        else:
            raise TypeError(f"added elements should be of type LegoItem or "
                            f"LegoComponent not {type(element)}")

    def children(self) -> Dict[str, List[LegoItem] | List[LegoComponent]]:
        return {'items': self.items, 'components': self.components}


# TODO: Adjust default output when printing an item or component